@@ -46,6 +46,8 @@ from geomapi.sessionnode import SessionNode
 
 import geomapi.utils as ut
 import geomapi.geometryutils as gt
+import geomapi.geometryutils2 as gt2
+
 
 def e57xml_to_nodes(e57XmlPath :str, **kwargs) -> List[PointCloudNode]:
     """Parse XML file that is created with E57lib e57xmldump.exe
@@ -250,7 +252,7 @@ def graph_to_nodes(graph : Graph,**kwargs) -> List[Node]:
     return nodelist
 
 
-def ifc_to_nodes(ifcPath:str, classes:str='ifcElement',**kwargs)-> List[BIMNode]:
+def ifc_to_nodes(ifcPath:str, classes:str='ifcElement',getGeometry=True,**kwargs)-> List[BIMNode]:
     """
     Parse ifc file to a list of BIMNodes, one for each ifcElement
 
@@ -266,7 +268,7 @@ def ifc_to_nodes(ifcPath:str, classes:str='ifcElement',**kwargs)-> List[BIMNode]
         selector = Selector()
         for ifcElement in selector.parse(ifc, classes): 
             try:   
-                node=BIMNode(ifcElement=ifcElement,ifcPath=ifcPath, **kwargs)
+                node=BIMNode(ifcElement=ifcElement,ifcPath=ifcPath,getGeometry=getGeometry, **kwargs)
                 if getattr(node,'mesh',None) is not None and len(node.mesh.triangles) !=0:
                     node.get_metadata_from_mesh()
             except:
@@ -293,28 +295,28 @@ def select_k_nearest_nodes(node:Node,nodelist:List[Node],k:int=10) -> List [Node
         List [Node]: 
     """
     #get node center
-    point=node.get_center()
+    if node.get_center() is not None:
+        point=node.center
+        #create pcd from nodelist centers
+        pcd = o3d.geometry.PointCloud()
+        array=np.empty(shape=(len(nodelist),3))
+        for idx,node in enumerate(nodelist):
+            node.get_center()
+            if getattr(node,'center',None) is not None: 
+                array[idx]=node.center   
+            else:
+                array[idx]=[-1000.0,-1000.0,-1000.0]
+        pcd.points = o3d.utility.Vector3dVector(array)
 
-    #create pcd from nodelist centers
-    pcd = o3d.geometry.PointCloud()
-    array=np.empty(shape=(len(nodelist),3))
-    for idx,node in enumerate(nodelist):
-        node.get_center()
-        if getattr(node,'center',None) is not None: 
-            array[idx]=node.center   
-        else:
-            array[idx]=[-1000.0,-1000.0,-1000.0]
-    pcd.points = o3d.utility.Vector3dVector(array)
+        #Create KDTree from pcd
+        pcdTree = o3d.geometry.KDTreeFlann(pcd)
 
-    #Create KDTree from pcd
-    pcdTree = o3d.geometry.KDTreeFlann(pcd)
+        #Find 200 nearest neighbors
+        [k1, idxList, _] = pcdTree.search_knn_vector_3d(point, k)
+        selectedNodeList=[node for idx,node in enumerate(nodelist) if idx in idxList]
 
-    #Find 200 nearest neighbors
-    [k1, idxList, _] = pcdTree.search_knn_vector_3d(point, k)
-    selectedNodeList=[node for idx,node in enumerate(nodelist) if idx in idxList]
-
-    if any(selectedNodeList):        
-        return selectedNodeList
+        if any(selectedNodeList):        
+            return selectedNodeList
     else:
         return None
 
@@ -332,28 +334,28 @@ def select_nodes_with_centers_in_radius(node:Node,nodelist:List[Node],r:float=0.
         List [Node]: 
     """
     #get node center
-    point=node.get_center()
+    if node.get_center() is not None:
+        point=node.center
+        #create pcd from nodelist centers
+        pcd = o3d.geometry.PointCloud()
+        array=np.empty(shape=(len(nodelist),3))
+        for idx,node in enumerate(nodelist):
+            node.get_center()
+            if getattr(node,'center',None) is not None: 
+                array[idx]=node.center 
+            else:
+                array[idx]=[-1000.0,-1000.0,-1000.0]
+        pcd.points = o3d.utility.Vector3dVector(array)
 
-    #create pcd from nodelist centers
-    pcd = o3d.geometry.PointCloud()
-    array=np.empty(shape=(len(nodelist),3))
-    for idx,node in enumerate(nodelist):
-        node.get_center()
-        if getattr(node,'center',None) is not None: 
-            array[idx]=node.center 
-        else:
-            array[idx]=[-1000.0,-1000.0,-1000.0]
-    pcd.points = o3d.utility.Vector3dVector(array)
+        #Create KDTree from pcd
+        pcdTree = o3d.geometry.KDTreeFlann(pcd)
 
-    #Create KDTree from pcd
-    pcdTree = o3d.geometry.KDTreeFlann(pcd)
+        #Find 200 nearest neighbors
+        [k1, idxList, _] = pcdTree.search_radius_vector_3d(point, r)
+        selectedNodeList=[node for idx,node in enumerate(nodelist) if idx in idxList]
 
-    #Find 200 nearest neighbors
-    [k1, idxList, _] = pcdTree.search_radius_vector_3d(point, r)
-    selectedNodeList=[node for idx,node in enumerate(nodelist) if idx in idxList]
-
-    if any(selectedNodeList):        
-        return selectedNodeList
+        if any(selectedNodeList):        
+            return selectedNodeList
     else:
         return None
 
@@ -373,24 +375,25 @@ def select_nodes_with_centers_in_bounding_box(node:Node,nodelist:List[Node],u:fl
         List [Node]
     """
     #get box source node
-    box=node.get_bounding_box()
-    box=gt.expand_box(box,u=u,v=v,w=w)
+    if node.get_bounding_box() is not None:
+        box=node.oriendtedBoundingBox
+        box=gt.expand_box(box,u=u,v=v,w=w)
 
-    # get centers
-    centers=np.empty((len(nodelist),3),dtype=float)
-    for idx,node in enumerate(nodelist):
-        centers[idx]=node.get_center()
+        # get centers
+        centers=np.empty((len(nodelist),3),dtype=float)
+        for idx,node in enumerate(nodelist):
+            centers[idx]=node.get_center()
 
-    #points are the centers of all the nodes
-    pcd = o3d.geometry.PointCloud()
-    points = o3d.utility.Vector3dVector(centers)
-    pcd.points=points
+        #points are the centers of all the nodes
+        pcd = o3d.geometry.PointCloud()
+        points = o3d.utility.Vector3dVector(centers)
+        pcd.points=points
 
-    # Find the nodes that lie within the index box 
-    idxList=box.get_point_indices_within_bounding_box(points)
-    selectedNodeList=[node for idx,node in enumerate(nodelist) if idx in idxList]
-    if any(selectedNodeList):        
-        return selectedNodeList
+        # Find the nodes that lie within the index box 
+        idxList=box.get_point_indices_within_bounding_box(points)
+        selectedNodeList=[node for idx,node in enumerate(nodelist) if idx in idxList]
+        if any(selectedNodeList):        
+            return selectedNodeList
     else:
         return None
 
@@ -410,19 +413,20 @@ def select_nodes_with_bounding_points_in_bounding_box(node:Node,nodelist:List[No
         List [Node]
     """
     #get box source node
-    box=node.get_bounding_box()
-    box=gt.expand_box(box,u=u,v=v,w=w)
+    if node.get_bounding_box() is not None:
+        box=node.oriendtedBoundingBox
+        box=gt.expand_box(box,u=u,v=v,w=w)
 
-    # get boxes nodelist
-    boxes=np.empty((len(nodelist),1),dtype=o3d.geometry.OrientedBoundingBox)
-    for idx,node in enumerate(nodelist):
-        boxes[idx]=node.get_bounding_box()
+        # get boxes nodelist
+        boxes=np.empty((len(nodelist),1),dtype=o3d.geometry.OrientedBoundingBox)
+        for idx,node in enumerate(nodelist):
+            boxes[idx]=node.get_bounding_box()
 
-    # Find the nodes of which the bounding points lie in the source node box
-    idxList=gt.get_box_inliers(box,boxes)
-    selectedNodeList=[node for idx,node in enumerate(nodelist) if idx in idxList]
-    if any(selectedNodeList):        
-        return selectedNodeList
+        # Find the nodes of which the bounding points lie in the source node box
+        idxList=gt.get_box_inliers(box,boxes)
+        selectedNodeList=[node for idx,node in enumerate(nodelist) if idx in idxList]
+        if any(selectedNodeList):        
+            return selectedNodeList
     else:
         return None
     
@@ -442,26 +446,28 @@ def select_nodes_with_intersecting_bounding_box(node:Node,nodelist:List[Node],u:
         List [Node]
     """
     #get box source node
-    box=node.get_bounding_box()
-    box=gt.expand_box(box,u=u,v=v,w=w)
+    if node.get_bounding_box() is not None:
+        box=node.oriendtedBoundingBox
+        box=gt.expand_box(box,u=u,v=v,w=w)
 
-    # get boxes nodelist
-    boxes=np.empty((len(nodelist),1),dtype=o3d.geometry.OrientedBoundingBox)
-    for idx,node in enumerate(nodelist):
-        boxes[idx]=node.get_bounding_box()
-    
-    # Find the nodes of which the bounding box itersects with the source node box
-    idxList=gt.get_box_intersections(box,boxes)
-    selectedNodeList=[node for idx,node in enumerate(nodelist) if idx in idxList]
-    if any(selectedNodeList):        
-        return selectedNodeList
+        # get boxes nodelist
+        boxes=np.empty((len(nodelist),1),dtype=o3d.geometry.OrientedBoundingBox)
+        for idx,node in enumerate(nodelist):
+            boxes[idx]=node.get_bounding_box()
+        
+        # Find the nodes of which the bounding box itersects with the source node box
+        idxList=gt.get_box_intersections(box,boxes)
+        selectedNodeList=[node for idx,node in enumerate(nodelist) if idx in idxList]
+        if any(selectedNodeList):        
+            return selectedNodeList
     else:
         return None
 
 def select_nodes_with_intersecting_meshes(node:Node,nodelist:List[Node]) -> List [Node]: 
     """Select the nodes of which the o3d.geometry.TriangleMeshes intersect
+    This method relies on trimesh and fcl libraries for collision detection
 
-    .. image:: ../../pics/selection_BB_mesh1.PNG
+    .. image:: ../../pics/collision_5.PNG
 
     Args:
         node (Node): source Node
@@ -471,17 +477,16 @@ def select_nodes_with_intersecting_meshes(node:Node,nodelist:List[Node]) -> List
         List [Node]: 
     """
     #get geometry source node
-    if node.get_geometry() and getattr(node,'mesh',None) is not None:
-        # get geometries nodelist
+    if getattr(node,'get_mesh',None) is not None and node.get_mesh() is not None: 
+        # get geometries nodelist        
         meshes=np.empty((len(nodelist),1),dtype=o3d.geometry.TriangleMesh)
-        for idx,node in enumerate(nodelist):
-            if node.get_geometry() and getattr(node,'mesh',None) is not None: 
-                meshes[idx]=node.mesh
+        for idx,testnode in enumerate(nodelist):
+            if getattr(testnode,'get_mesh',None) is not None and testnode.get_mesh() is not None: 
+                    meshes[idx]=testnode.mesh
 
         # Find the nodes of which the geometry itersects with the source node box
-        idxList=gt.mesh_collisions_trimesh(node.mesh,meshes)
+        idxList=gt2.mesh_collisions_trimesh(node.mesh,meshes)
         selectedNodeList=[node for idx,node in enumerate(nodelist) if idx in idxList]
         if any(selectedNodeList):        
             return selectedNodeList
-    
     return None
\ No newline at end of file
